"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.treeApi = void 0;
const internal_1 = require("@fluidframework/core-utils/internal");
const index_js_1 = require("../feature-libraries/index.js");
const index_js_2 = require("../simple-tree/index.js");
const index_js_3 = require("../util/index.js");
const schematizingTreeView_js_1 = require("./schematizingTreeView.js");
const treeView_js_1 = require("./treeView.js");
/**
 * The `Tree` object holds various functions for interacting with {@link TreeNode}s.
 * @public
 */
exports.treeApi = {
    ...index_js_2.treeNodeApi,
    runTransaction(treeOrNode, transaction, preconditions = []) {
        if (treeOrNode instanceof schematizingTreeView_js_1.SchematizingSimpleTreeView) {
            const t = transaction;
            runTransaction(treeOrNode.checkout, () => t(treeOrNode.root), preconditions);
        }
        else {
            const node = treeOrNode;
            const t = transaction;
            const context = (0, index_js_2.getFlexNode)(node).context;
            (0, internal_1.assert)(context instanceof index_js_1.Context, 0x901 /* Unsupported context */);
            const treeView = treeView_js_1.contextToTreeView.get(context) ??
                (0, index_js_3.fail)("Expected view to be registered for context");
            runTransaction(treeView.checkout, () => t(node), preconditions);
        }
    },
    contains(parent, child) {
        let toCheck = child;
        while (toCheck !== undefined) {
            if (toCheck === parent) {
                return true;
            }
            toCheck = exports.treeApi.parent(toCheck);
        }
        return false;
    },
};
// TODO: Add more constraint types here
function runTransaction(checkout, transaction, preconditions) {
    checkout.transaction.start();
    for (const constraint of preconditions) {
        switch (constraint.type) {
            case "nodeInDocument": {
                const node = (0, index_js_2.getFlexNode)(constraint.node);
                (0, internal_1.assert)(node.treeStatus() === index_js_1.TreeStatus.InDocument, 0x90f /* Attempted to apply "nodeExists" constraint when building a transaction, but the node is not in the document. */);
                checkout.editor.addNodeExistsConstraint(node.anchorNode);
                break;
            }
            default:
                (0, internal_1.unreachableCase)(constraint.type);
        }
    }
    let result;
    try {
        result = transaction();
    }
    catch (e) {
        // If the transaction has an unhandled error, abort and rollback the transaction but continue to propagate the error.
        checkout.transaction.abort();
        throw e;
    }
    if (result === "rollback") {
        checkout.transaction.abort();
    }
    else {
        checkout.transaction.commit();
    }
}
//# sourceMappingURL=treeApi.js.map